public class StarGalaxy extends Galaxy {

/*
**********************************************************************
Class Variables
**********************************************************************
*/

   Star[] stars;
   int nstars;
   double galtheta,galphi,diskSize;

/*
**********************************************************************
Constructor
**********************************************************************
*/

   StarGalaxy(double massin, double ahaloin, double vhaloin, double rthaloin,
              double dsizein, double thetain, double phiin, int n) {

      int istar;

      galmass=massin;
      ahalo=ahaloin;
      vhalo=vhaloin;
      rthalo=rthaloin;
      diskSize=dsizein;
      galpos.x=0.0;
      galpos.y=0.0;
      galpos.z=0.0;
      galvel.x=0.0;
      galvel.y=0.0;
      galvel.z=0.0;
      galtheta=thetain;
      galphi=phiin;

      nstars=n;
      stars = new Star[nstars];
      for (istar=0; istar<nstars; istar++) {
         stars[istar] = new Star();
      } // istar for loop

      InitStars();

   } //constructor

/*
**********************************************************************
setAngles  
**********************************************************************
*/

   public void setAngles(double thetain, double phiin) {
      galtheta=thetain;
      galphi=phiin;
   }

/*
**********************************************************************
setNumstars
**********************************************************************
*/

   public void setNumstars(int nstarin) {
      nstars=nstarin;
   }

/*
**********************************************************************
setPosvel  
**********************************************************************
*/

   public void setPosvel(Triple posin, Triple velin) {

      int istar;

      galpos = posin;
      galvel = velin;
      for (istar=0; istar<nstars; istar++) {
         stars[istar].pos = Triple.AddTriple(stars[istar].pos, galpos);
         stars[istar].vel = Triple.AddTriple(stars[istar].vel, galvel);
      } // istar for loop


   } //setPosvel


/*
**********************************************************************
MoveStars  
**********************************************************************
*/

   public void MoveStars(double dtime) {

      Triple newpos = new Triple();
      Triple newvel = new Triple();
      int istar;


      for (istar=0; istar<nstars; istar++) {

         stars[istar].pos.x = stars[istar].pos.x + 
                    stars[istar].vel.x * dtime + 
                    0.5f * stars[istar].acc.x * dtime*dtime;
         stars[istar].pos.y = stars[istar].pos.y + 
                    stars[istar].vel.y * dtime + 
                    0.5f * stars[istar].acc.y * dtime*dtime;
         stars[istar].pos.z = stars[istar].pos.z + 
                    stars[istar].vel.z * dtime + 
                    0.5f * stars[istar].acc.z * dtime*dtime;

         stars[istar].vel.x = stars[istar].vel.x + 
                    stars[istar].acc.x * dtime;
         stars[istar].vel.y = stars[istar].vel.y + 
                    stars[istar].acc.y * dtime;
         stars[istar].vel.z = stars[istar].vel.z + 
                    stars[istar].acc.z * dtime;

      } //istar for loop

   } //MoveStars

/*
**********************************************************************
InitStars  
**********************************************************************
*/

   public void InitStars() {

      boolean bad;
      int istar;
      double xtry,ytry,ztry,rtry,xrot,yrot,zrot,vcirc;
      double vxtry,vytry,vztry,vxrot,vyrot,vzrot;
      double cosphi,sinphi,costheta,sintheta;

      cosphi=Math.cos(galphi);
      sinphi=Math.sin(galphi);
      costheta=Math.cos(galtheta);
      sintheta=Math.sin(galtheta);

      for (istar=0; istar<nstars; istar++) {

         bad=true;
         do {
           xtry=diskSize*(1.-2.*Math.random());
           ytry=diskSize*(1.-2.*Math.random());
           rtry=Math.sqrt(xtry*xtry+ytry*ytry);
           if (rtry < diskSize) bad=false;
         } while (bad);
         ztry=0.0;

         xrot=xtry*cosphi + ytry*sinphi*costheta + ztry*sinphi*sintheta;
         yrot=-xtry*sinphi + ytry*cosphi*costheta + ztry*cosphi*sintheta;
         zrot=-ytry*sintheta + ztry*costheta;

         stars[istar].pos.x = xrot + galpos.x;
         stars[istar].pos.y = yrot + galpos.y;
         stars[istar].pos.z = zrot + galpos.z;

         vcirc=Math.sqrt(this.InteriorMass(rtry)/rtry);

         vxtry=-vcirc*ytry/rtry;
         vytry=vcirc*xtry/rtry;
         vztry=0.0;

         vxrot=vxtry*cosphi + vytry*sinphi*costheta + vztry*sinphi*sintheta;
         vyrot=-vxtry*sinphi + vytry*cosphi*costheta + vztry*cosphi*sintheta;
         vzrot=-vytry*sintheta + vztry*costheta;


         stars[istar].vel.x = vxrot + galvel.x;
         stars[istar].vel.y = vyrot + galvel.y;
         stars[istar].vel.z = vzrot + galvel.z;

         stars[istar].acc.x = 0;
         stars[istar].acc.y = 0;
         stars[istar].acc.z = 0;

      } // istar for loop

   } // InitGalStars

/*
**********************************************************************
scaleMass
**********************************************************************
*/

   public void scaleMass(double massFact) {

      diskSize=2.5*Math.pow(massFact,0.5);
      super.scaleMass(massFact);

   } // scaleMass   

}
